%% 
% Polynomial Basis Pursuit based on l1-minimization
% with l2 bound on the noise
%
% by F. Lauer, 2013. 
%
%
%  [x, phi, yhat, t] = polybpL1noiseL2(Q,y,n,deg, epsilon, iters, verbose)
%
%	Compute x = phi(1:n)
%
%	with 	phi = argmin  || Phi ||_1
%
%					s.t. || A Phi + b - y ||_2 < epsilon 
%
%
% Inputs (N = nb of equations; M = dim(phi) ):
%	
%	Q 		: N-by-(M+1) matrix of polynomial coefficients 
%				Q(:,1) = b ;  Q(:,2:end) = A
%   y 		: N-by-1 vector of measurements
%   n		: number of variables = dim(x)
%   deg 	: maximal degree of the polynomials 
%   epsilon	: bound on the l2-norm of the error 
%   iters   : number of iterations for reweighting 
%	verbose : print info at each iteration if true
%
% Outputs:
%	x   	: n-vector estimate of x
%	phi 	: M-vector solution to the optimization problem (estimates of the monomials)
%	yhat	: N-vector approximation of y as yhat = A phi + b
%	pattern : M-vector equal to abs(phihat)
%
%
% The columns in Q must be ordered such that Q(:,k) contains the 
% coefficients associated to the monomial x^(alpha_k) = prod(x.^(alpha(k,:)'))
% with alpha = multiindex(n, deg). 
%
function [xhat, phihat, yhat, t] = polybpL1noiseL2(Q,y,n,degree, epsilon, Niters, verbose)

if nargin < 5
	epsilon = 0.1;
end

if nargin < 6
	Niters = 10;
end

if nargin < 7
	verbose = false; 
end


N = size(Q,1);

alpha = multiindex(n, degree);
dim_alpha = size(alpha,1);
dim = dim_alpha - 1;

dim_uj = sum(alpha(:,1)~=0); 

tj = dim + dim_uj*n;
start_u = dim;

nvars = 2*dim + N + 1; 

Aeq = [Q(:,2:end) spalloc(N,dim,0) speye(N,N) spalloc(N,1,0)];
Beq = y - Q(:,1) ;

Aineq = sparse([speye(dim) -speye(dim) spalloc(dim,N+1,0) ; -speye(dim) -speye(dim) spalloc(dim,N+1,0)]);
Bineq = zeros(2*dim,1);

% Cone constraint ||e|| < error
prob.cones{1}.type = 'MSK_CT_QUAD';	
prob.cones{1}.sub  = [nvars, nvars-N:nvars-1];

% Optimize with MOSEK

% constraints
prob.a  = sparse([Aeq;Aineq]);

prob.blc = [Beq',-inf*ones(1,length(Bineq))];
prob.buc = [Beq',Bineq'];

% Bounds on variables
UB = [inf*ones(nvars-1,1); epsilon];
LB = [-inf*ones(dim,1); zeros(dim,1); -inf*ones(N,1); 0];

prob.blx = LB';
prob.bux = UB';

% Solve L1-min

W = ones(dim,1);
phihat = zeros(dim,1);

for iter = 1:Niters
	% reweighted cost 
	lincost = [zeros(dim,1); W; zeros(N+1,1)];

	prob.c = sparse(lincost);

	[r, res] = mosekopt('minimize echo(0)',prob);
	sol = res.sol.itr.xx;

	if sum(sol(1:dim) == phihat)==dim , break, end
	
	phihat = sol(1:dim);
	xhat = phihat(1:n);
	
	if verbose
		sparsityFactorPhi = sum(abs(phihat) < 1e-6) / dim

		xhat
	
		SumSquaredError = checksol(xhat,Q,y,n,degree)
	end
	
	yhat = Q * [1;phihat];

	t = sol(dim+1: 2*dim);

	W = 1./(t + 0.001);
end
