%% 
% Greedy algorithm for finding sparse solutions of a polynomial system 
%
% by F. Lauer, 2013. 
%
%  [x, S, err, phi] 
%		= polygreedy(Q, y, n, deg, epsilon, verbose)
%
%	Compute x = phi(1:n)
%
%	with 	phi an approximate solution to
%
%				min sum_j || { W_j Phi }_j=1^n ||_0
%
%					s.t. || A Phi + b - y ||_2 < epsilon 
%
% Inputs (N = nb of equations; M = dim(phi) ):
%	
%	Q 		: N-by-(M+1) matrix of polynomial coefficients 
%				Q(:,1) = b ;  Q(:,2:end) = A
%   y 		: N-by-1 vector of measurements
%   n		: number of variables = dim(x)
%   deg 	: maximal degree of the polynomials 
%   epsilon	: bound on the l2-norm of the error 
%	verbose : print info at each iteration if true
%
% Outputs:
%	x   	: n-vector estimate of x
%	S		: support of x 
%	err		: remaining error || A phi + b - y ||_2^2
%	phi 	: M-vector with estimates of the monomials
%
% The columns in Q must be ordered such that Q(:,k) contains the 
% coefficients associated to the monomial x^(alpha_k) = prod(x.^(alpha(k,:)'))
% with alpha = multiindex(n, deg). 
%
function [xhat, S, err, phihat] = polygreedy(Q,y,n,degree, epsilon, verbose)
	
	if nargin < 6
		verbose = false; 
	end

	if nargin < 5 
		epsilon = 0.1; % tol on remaining error for A*phi -y
	end

	epsilon2 = epsilon .* epsilon;

	N = size(Q,1);

	alpha = multiindex(n, degree);
	dim_alpha = size(alpha,1);
	dim = dim_alpha - 1;

	S = []; 
	notS = 1:n;

	yb = y - Q(:,1);

	err = sum(yb.^2);
	
	while ( (err > epsilon2) && (length(S) < n) )

		dim_j = length(support([S,notS(1)],alpha));
		phi_j = zeros(dim_j,n);
		
		
		% For each variable not in support of solution (S),
		%	minimize the error with support = S + this variable
		e = zeros(1,length(notS));
		i=1;
		for j=notS
	
			A = Q(:,support([S, j], alpha) ); 
			phi_j(:,j) = A \ yb;

			e(i) = sum(( A * phi_j(:,j)  - yb).^2);
			i = i +1;
		end
	
		% Find best variable with minimum error
		[err, k] = min(e);
	
		% add variable to support of solution
		S = [S, notS(k)];
		notS(k) = [];
	end	
	
	phihat = zeros(dim_alpha,1);
	phihat( support(S, alpha) ) = phi_j(:,S(end));
	phihat(1) = [];
	
	xhat = phihat(1:n);
	
end

function supp = support(idx, alpha) 
	notidx = (1:size(alpha,2));
	notidx(idx) = [];
	supp = find(sum(alpha(:,notidx),2) == 0); 
	supp(1) = [];
	
end


